﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        private ObservableCollection<IShape> shapes = new ObservableCollection<IShape>();
        private static Random random = new Random();

        public Form1()
        {
            InitializeComponent();
            pictureBox1.Image = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            shapes.CollectionChanged += Shapes_CollectionChanged;
        }

        private void Shapes_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            Redraw();
        }

        public void Redraw()
        {
            using (var g = Graphics.FromImage(pictureBox1.Image))
            {
                foreach (var shape in shapes)
                {
                    shape.Draw(g);
                }
            }
            pictureBox1.Invalidate();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            shapes.Add(new Line
            {
                Pen = Pens.Red,
                Start = new PointF(random.Next(pictureBox1.Width), random.Next(pictureBox1.Height)),
                End = new PointF(random.Next(pictureBox1.Width), random.Next(pictureBox1.Height))
            });
        }

        private void pictureBox1_SizeChanged(object sender, EventArgs e)
        {
            pictureBox1.Image = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            Redraw();
        }

        private IShape FindShape(PointF p)
        {
            // Reverse search order because we draw from bottom to top, but we need to hit-test from top to bottom.
            foreach (var shape in shapes.Reverse())
            {
                if (shape.IsHit(p))
                    return shape;
            }
            return null;
        }

        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            var shape = FindShape(e.Location);
            if (shape != null)
            {
                shape.Pen = Pens.Blue;
                Redraw();
            }
        }
    }
}
